#include <stdio.h>
#include <stdlib.h>
#include <math.h>

typedef struct point
{
	double x, y;
} point_t;

typedef struct triangle
{
	point_t a, b, c;
	struct triangle* next;
} triangle_t;

triangle_t *create_triangle(point_t *a, point_t *b, point_t *c)
{
	triangle_t *t = NULL;

	if (a == NULL || b == NULL || c == NULL) 
	{
		printf("Error: illegal argument");
		return NULL;
	}

	t = (triangle_t*) malloc(sizeof(triangle_t));
	if (t != NULL)
	{
		t->a = *a;
		t->b = *b;
		t->c = *c;
		t->next = NULL;
	}

	return t;
}

triangle_t *addTriangle(triangle_t *mesh, triangle_t *t)
{
	if (t != NULL)
	{
		t->next = mesh;
		return t;
	}
	return mesh;
}

double triangleArea(triangle_t *t)
{
	return fabs((t->a.x * (t->b.y - t->c.y) +
				 t->b.x * (t->c.y - t->a.y) +
				 t->c.x * (t->a.y - t->b.y)) / 2);
}

double meshArea(triangle_t *mesh)
{
	if (mesh == NULL)
		return 0;
	return triangle_area(mesh) + mesh_area(mesh->next);
}

void free_mesh(triangle_t *head)
{
	if (head == NULL)
		return;
	free_mesh(head->next);
	free(head);
}

int main()
{
	triangle_t *mesh = NULL;

	point_t t1_vertices[3] = { {15, 15} , {23, 30} , {50, 25} };
	point_t t2_vertices[3] = { {15, 15} , {45, 10} , {50, 25} };
	point_t t3_vertices[3] = { {45, 10} , {55, 10}  , {50, 2} };

	triangle_t *t1 = create_triangle(&t1_vertices[0], &t1_vertices[1], &t1_vertices[2]);
	triangle_t *t2 = create_triangle(&t2_vertices[0], &t2_vertices[1], &t2_vertices[2]);
	triangle_t *t3 = create_triangle(&t3_vertices[0], &t3_vertices[1], &t3_vertices[2]);

	mesh = add_triangle(mesh, t1);
	mesh = add_triangle(mesh, t2);
	mesh = add_triangle(mesh, t3);
	printf("1. triangle area = %g\n", triangle_area(t1));
	printf("2. triangle area = %g\n", triangle_area(t2));
	printf("3. triangle area = %g\n", triangle_area(t3));
	printf("mesh area = %g\n", mesh_area(mesh));

	free_mesh(mesh);

	return 0;
}

